% StroopStimResponse.m
%
% This program presents stroop like stimuli and record keypress response.
% Written by YW, 2022

try
    subid='1';
    subname='ZhangSan';
    Screen('Preference', 'Verbosity', 0);
    Screen('Preference', 'SkipSyncTests',1);
    Screen('Preference', 'VisualDebugLevel',0);
    
    % predefine parameters
    BlockNum=1;% how may block in this experiment
    StimDuration=2000;% 2000ms
    ISImin=1000;% the minmum interstimulus interval (ISI)
    ISImax=1500;% the maxmum interstimulus interval (ISI)
    TxtSize=100;% the size of the texts
    TxtFont='Arial'; % the font of the texts
    folder_path = 'D:\Unit7m-file\picture'; 
    file_names = dir(fullfile(folder_path, '*.jpg')); % ȡļ.jpgʽͼƬļ
    images = cell(size(file_names));
    for i = 1:length(file_names)
        images{i} = imread(fullfile(folder_path, file_names(i).name));
    end
    
    %%%Ȼ󣬶ͼƬԤСһ
    processed_images = cell(size(images));
    for i = 1:length(images)
        processed_images{i} = imresize(images{i}, [680 680]); 
    end
    StimColor={[255 0 0];[0 0 255]};
    count=0;
    for i=1:length(processed_images)
        for j=1:length(StimColor)
            count=count+1;
            Stim(count).processed_images =processed_images{i}; % stimulus word
            Stim(count).color=StimColor{j};% stimulus color
        end
    end
    
    % to preset variables 'results' for recording data
    results.processed_images= NaN*ones(BlockNum,length(Stim));
    results.color = NaN*ones(BlockNum,length(Stim));
    results.response = NaN*ones(BlockNum,length(Stim));
    results.reactiontime = NaN*ones(BlockNum,length(Stim));
    
    % open window
    screenNum=0;
    [wPtr,rect]=Screen('OpenWindow',screenNum);
    HideCursor;
    black=BlackIndex(wPtr);
    white=WhiteIndex(wPtr);
    monitorFlipInterval=Screen('GetFlipInterval', wPtr);
    hz=FrameRate(wPtr);
    StimDuration=round((StimDuration/1000)*hz); % the number of frames for stimulusduration
    ISIrange=round(((ISImax-ISImin)/1000)*hz);
    ISImin=round((ISImin/1000)*hz);
    ISI=round(rand(BlockNum,length(Stim))*ISIrange+ISImin);% the number of frames for neach trial's ISI in a range of ISImin and ISImax
    
    % blank the Screen
    starttime=Screen('Flip',wPtr); % collect the time for the first flip with starttime
    
    
    for ii=1:BlockNum
        StimRand=Stim(randperm(length(Stim))); % randomize all stimuli
        
        for jj=1:length(Stim)

            Screen('FillRect',wPtr,StimRand(jj).color,[(rect(3)/2 - 400) (rect(4)/2 - 400)  (rect(3)/2 + 400) (rect(4)/2 + 400)]);
            textureIndex=Screen('MakeTexture', wPtr, StimRand(jj).processed_images);%%%'MakeTexture'ͼƬwPtrȡdisgustface textureIndex˵
            ret=Screen('PreloadTextures', wPtr, textureIndex);
            Screen('DrawTexture', wPtr, textureIndex );%%%ͼ
            Screen(wPtr, 'Flip');
            keys = {};
            RT = [];
            ListenChar(2);
            nKeys = 0;
            results.color(ii,jj)=find(StimRand(jj).color~=0);
            while GetSecs-starttime < StimDuration*monitorFlipInterval
                %see if a key is pressed
                [ keyIsDown, timeSecs, keyCode ] = KbCheck;
                if keyIsDown %a key is down: record the key and time pressed
                    nKeys = nKeys+1;
                    RT(nKeys) = timeSecs-starttime;
                    keys{nKeys} = KbName(keyCode);
                    %clear the keyboard buffer
                    while KbCheck; 
                    end
                    
                end
            end

        starttime=Screen('Flip', wPtr);% time in secs
        
        %%% save keypress response
        if StimRand(jj). processed_images == processed_images{1}
            results.word(ii,jj)=1; 
        elseif StimRand(jj). processed_images == processed_images{2}
            results.word(ii,jj)=2;
        elseif StimRand(jj). processed_images == processed_images{3}
            results.word(ii,jj)=3;
        elseif StimRand(jj). processed_images == processed_images{4}
            results.word(ii,jj)=4;
        elseif StimRand(jj). processed_images == processed_images{5}
            results.word(ii,jj)=5;
        elseif StimRand(jj). processed_images == processed_images{6}
            results.word(ii,jj)=6;
        elseif StimRand(jj). processed_images == processed_images{7}
            results.word(ii,jj)=7;
        elseif StimRand(jj). processed_images == processed_images{8}
            results.word(ii,jj)=8;
        end
        results.color(ii,jj)=find(StimRand(jj).color~=0); % color 1=red,2=green, 3=blue
        
        if ~isempty(RT)
            results.reactiontime(ii,jj)=RT;
            results.key{ii,jj}=keys{1};
            
            if (results.key{ii,jj}=='f' && results.color(ii,jj)==1)...
                    ||(results.key{ii,jj}=='k' && results.color(ii,jj)==3)
                results.response(ii,jj) = 1;% f for color red, j for color green, k for color blue are correct
                
                
            else
                results.response(ii,jj) = 0;% incorrect
                
            end
        end
    end
end
ISIlast=round(rand(1)*ISIrange+ISImin); % the last trial's ISI
% blank the screen and wait a second
Screen('FillRect',wPtr,white);
Screen(wPtr, 'Flip', starttime+(ISIlast*monitorFlipInterval));

% save the datafile to disc for each participant
datafile=['StroopData_Sub' subid '_' subname];
path=pwd;
save ([path '\Data\' datafile '.mat'], 'results'); % save as mat file

Screen('CloseAll');
ShowCursor
catch
    Screen('CloseAll')
    ListenChar(0);
    ShowCursor
    rethrow(lasterror)
 end
